/*----------------------------------------------------------------------*
 * Env.h
 *
 * Michael Geikowski, 14.7.93 
 *
 * Definiert die Klasse Environment zur Verwaltung des TLA+ Umgebungs-
 * modells
 *----------------------------------------------------------------------*/

#ifndef ENV_H
#define ENV_H

#include "list.h"
#include "sort.h"
#include "symlist.h"


class Environment : public List
//=============================
	// Ein Objekt der Klasse Environment verwaltet einen Stack von Rahmen
        // des Umgebungsmodells eines Moduls. Am weitesten unten im Stack liegt
        // immer der globale Rahmen. Darauf koennen weitere Rahmen gestapelt
        // werden. Diese sind entweder globale Rahmen von Untermodulen oder
        // lokale Rahmen des Moduls oder seiner Untermodule. Jeder Rahmen
        // enthaelt eine Menge von Bezeichnern, Default-Sorten sowie einen
        // Modulnamen und -typ. Die Bezeichner und der Modulname werden durch
        // Symbollistenelemente beschrieben.
        // Zusaetzlich wird der Name der Datei vermerkt, in der das Modul
        // enthalten ist (sollte in der Regel mit dem Modulnamen
        // uebereinstimmen).
	// Zur Behandlung von Ausnahmen der Regel `declare before use' wird
        // eine UnknownID-Umgebung eingesetzt, in der Bezeichner gesammelt
	// werden, die eventuell zu einem spaeteren Zeitpunkt noch definiert
	// werden koennen. Fuer diese Umgebung sind einge spezielle Funktionen
        // noetig. Ansonsten sind alle Funktionen fuer eine normale Umgebung
        // ebenfalls fuer eine UnknownID-Umgebung gueltig (wenn auch nicht
        // alle sinnvoll sind).
 {
 public:

   Environment(Sort_Type defSort = s_statefct, Module_Type t = mt_temporal,
               char* filename = 0                                         );
 //-------------------------------------------------------------------------
	// Neue Umgebung mit Dateiname sowie Default-Sorte und ModulTyp fuer
        // den globalen Rahmen.

   ~Environment();
 //---------------

   void Pop();
 //-----------
 	// Lokalen Rahmen auf Top des Stacks loeschen.

   void Push();
 //------------
 	// Neuen leeren lokalen Rahmen auf Top des Stacks schaffen.
        // Default-Sorten, Modulname und -typ werden vom vorigen Top Rahmen
        // uebernommen.

   void Append(SymListElem* e);
 //----------------------------
	// Fuegt den durch `e' beschriebenen Bezeichner in den Top Rahmen
        // ein bzw. erzeugt eine Fehlermeldung, falls der Bezeichner in
        // Konflikt steht mit einem bereits definierten oder importierten/
        // includierten Bezeichner. Bezeichner duerfen mehrfach importiert/
        // includiert werden, aber nicht mehrfach definiert oder definiert
        // und importiert/includiert werden.

   void AppendInEnclosingFrame(SymListElem* e);
 //--------------------------------------------
  	// Wie Append fuer Symbollistenelemente. Nur wird der Bezeichner
	// in den den Top Rahmen umfassenden Rahmen eingefuegt.

   void Append(SymList *slist, int isIncludeList=0, SymListElem* Import=0);
 //------------------------------------------------------------------------
 	// Wie Append fuer Symbollistenelemente. Nur werden hier gleich
        // mehrere Bezeichner, die in einer Liste `sList' gespeichert sind,
        // auf einmal in den Top Rahmen eingefuegt.
	// Ist `sList' eine Liste includierter Bezeichner, so ist
        // `isIncludeList' auf 1 zu setzen, damit Bezeichner der Klasse
        // "Annahme" in Bezeichner der Klasse "Theorem" gewandelt werden.
	// Ist `sList' eine Liste includierter/importierter Bezeichner, so ist
        // `Import' auf das Symbollistenelement zu setzen, das den Namen
        // des zu importierenden/includierenden Moduls im Quelltext beschreibt,
        // um eine eventuelle Fehlermeldung ausgeben zu koennen.

   void StartSubModule(Sort_Type defSort, Module_Type t);
 //------------------------------------------------------
	// Legt neuen globalen Rahmen fuer ein Untermodul mit Modultyp `t' und
        // Default-Sorte `defSort' auf dem Top des Stacks an.

   void AppendAndPopSubModule();
 //-----------------------------
 	// Loescht einen globale Rahmen eines Untermoduls vom Top des Stacks.
        // Hierdurch werden die Bezeichner des Top Rahmens in den
        // umfassenden Rahmen exportiert. Hat das Untermodul einen Namen,
        // so wird dieser als Qualifizierer in den umfassenden Rahmen
        // eingefuegt und an die Qualifiziererlisten aller qualifizierbaren
        // exportierten Bezeichner angehaengt. Konflikte werden wie bei
        // Append fuer Symbollistenelemente behandelt.

   void AppendUnknown(SymList* s);
 //-------------------------------
        // Fuegt einen (noch) undefinierten Bezeichner in den Top Rahmen einer
        // UnknownID-Umgebung ein. Dabei werden Konflikte zwischen Bezeichnern
        // bewusst ignoriert, da spaeter fuer jede Verwendung eines Bezeichners
        // die Beziehungen zu weiteren Verwendungen nachgetragen werden muessen.
        // Der einzufuegende Bezeichner ist gegeben durch eine Liste `s' aller
        // seiner Qualifizierer, des eigentlichen Bezeichners und aller
        // Record-Komponenten-Namen. Die Folgen der Sorten der Parameterlisten
        // des Bezeichners und seiner Qualifizierer sind --- wenn
        // Parameterlisten existieren --- als Sorten bei den entsprechenden
        // Symbollistenelementen eingetragen.

   void AppendAndPopUnknownIDs(int isHeader, Environment *Env);
 //------------------------------------------------------------
	// Top Rahmen der UnknownID-Umgebung wird geloescht. Die Bezeichner
        // dieses Rahmen werden daraufhin ueberprueft, ob sie inzwischen
        // gebunden wurden, d.h. ob sie im Top Rahmen von `Env'
        // eingetragen sind. Alle Bezeichner des Top Rahmen, die nicht
        // gebunden werden konnten, werden in den Rahmen eingetragen, der
        // den Top Rahmen umfasst. Ausnahme: der umfassende Rahmen ist der
        // globale Rahmen und `isHeader' ist False. Im letzten Fall koennen die
        // Bezeichner nicht mehr gebunden werden und es werden entsprechende
        // Fehlermeldungen erzeugt.
        // Bezeichner, die gebunden werden konnten, werden mit ihren letzten
        // bzw. naechsten Verwendungen und der Stelle ihrer Definition
        // verkettet.

   void PopUnknownIDs();
 //---------------------
	// Top Rahmen der UnknownID-Umgebung wird geloescht. Die Bezeichner
        // dieses Rahmen werden vorher in den Rahmen eingetragen, der den
        // Top Rahmen umfasst.

   void AppendRestUnknownIDs(Environment *Env);
 //--------------------------------------------
	// Die Bezeichner des top Rahmen der UnknownID-Umgebung werden
        // daraufhin ueberprueft, ob sie in Env gebunden sind. Fuer alle
        // Bezeichner, die nicht gebunden sind werden entsprechende
        // Fehlermeldungen erzeugt.
        // Bezeichner, die gebunden werden konnten, werden mit ihren letzten
        // bzw. naechsten Verwendungen und der Stelle ihrer Definition
        // verkettet.

   SymListElem* Lookup(SymListElem *e, SymList *qual);
 //---------------------------------------------------
	// Sucht einen Bezeichner `e' mit Qualifiziererliste `qual' im 
        // Umgebungsmodell. Gesucht wird vom der Top Rahmen an abwaerts.
        // Das erste Auftreten wird gemeldet. Ergibt NULL, wenn Bezeichner
        // nicht vorhanden.

   SymListElem* UseOfID(SymList* s);
 //---------------------------------
	// Ein Bezeichner `s' wird im Umgebungsmodell gesucht. Wird er
	// gefunden, so wird er mit den weiteren Verwendungen sowie der
	// Stelle der Definition verknuepft. Die Stelle der Definition wird
	// zurueckgegeben. Wird er nicht gefunden, wird eine Fehlermeldung
	// erzeugt.
	// Der Bezeichner kann qualifiziert und mit Record-Komponenten-Namen
	// behaftet sein. Er wird hier deshalb als Liste von normalID's
	// uebergeben.

   SymList* Export(SymList *exportlist);
 //-------------------------------------
	// Liefert die Liste aller zu exportierender Bezeichner einer
        // Umgebung. Mit `exportlist' wird die (evtl. leere) Liste aller
        // explizit zu exportierenden Bezeichner angegeben.
        // Alle Bezeichner der Exportliste werden daraufhin ueberprueft, ob
        // sie inklusive ihrer Qualifizierer in der Umgebung bekannt sind.

   Sort_Type GetDefaultParSort();
 //------------------------------
	// Default Parameter Sorte des Top Rahmen ausgeben (initial gleich
        // der Default Sorte)

   Sort_Type GetDefaultOpSort();
 //-----------------------------
	// Default Operator Sorte des Top Rahmen ausgeben (initial gleich
        // der Default Sorte)

   Sort_Type GetDefaultSort();
 //---------------------------
	// Default Sorte des Top Rahmen ausgeben

   void SetDefaultParSort(Sort_Type defSort);
 //------------------------------------------
 	// Default Parameter Sorte des Top Rahmen aendern

   void SetDefaultOpSort(Sort_Type defSort);
 //-----------------------------------------
  	// Default Operator Sorte des Top Rahmen aendern

   void SetDefaultSort(Sort_Type defSort);
 //---------------------------------------
 	// Default Sorte des Top Rahmen aendern
        
   Module_Type GetModuleType();
 //----------------------------
	// Modultyp des Top Rahmen ausgeben

   void SetModuleName(SymListElem *e);
 //-----------------------------------
	// Modulname des Top Rahmen aendern

   SymListElem* GetModuleName();
 //-----------------------------
	// Modulname des Top Rahmen ausgeben

   int Recover();
 //--------------
	// Umgebung bis zum ersten globalen Rahmen eines Untermoduls oder bis
        // zum globalen Rahmen des Gesamtmoduls poppen.



 private:

   char* actfilename;
 };

#endif
